/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function() {
    var svgBoard = document.querySelector("#svg-board"),
        svgContainer = document.querySelector("#svg-container"),
        shapeElements = document.querySelectorAll(".shape-element"),
        SVG_TAG_RECT = "<rect width='50' height='50' style='' />",
        SVG_TAG_RECTRND = "<rect rx='17.5' ry='17.5' width='50' height='50' style='' />",
        SVG_TAG_ELLIPSE = "<ellipse cx='25' cy='25' rx='25' ry='25' style='' />",
        SVG_TAG_TRIANGLE = "<polygon points='25,0 0,50 50,50' style='fill-rule:nonzero;' />",
        SVG_TAG_STAR = "<polygon points='25,0 10,50 50,19 0,19 40,50' style='fill-rule:nonzero;' />",
        customIndex = 0,
        customColor = "",
        customSizeTemp = 0,
        customSize = 0,
        customShapeTemp = null,
        customShape = null,
        drawPath = {},
        isMoving = 0,
        isPopup = 0;

    /**
     * Inserts a new SVG element to a SVG board in the HTML document
     * Since SVG elements cannot be added directly using the innerHTML method a workaround function is required
     * @private
     * @param {String} boardId - the id of the SVG board where the new element will be inserted to
     * @param {String} elemId - the id of the new SVG element to be inserted
     * @param {String} shapeId - the id of the shape to be created
     * @param {Number} shapeSize - the size of the shape
     * @param {Number} translateX - the x-coordinate of the shape
     * @param {Number} translateY - the y-coordinate of the shape
     */
    function insertSvgShape(boardId, elemId, shapeId, shapeSize, translateX, translateY) {
        var tempDiv = document.createElement("div"), // Create a dummy div element
            svgFragment = "",
            shapeTag = "",
            colorTag = "",
            tempTag = "";

        // Construct the tag for the new shape element
        switch (shapeId) {
            case "select-rect":
                shapeTag = SVG_TAG_RECT;
                break;
            case "select-rectrnd":
                shapeTag = SVG_TAG_RECTRND;
                break;
            case "select-ellipse":
                shapeTag = SVG_TAG_ELLIPSE;
                break;
            case "select-triangle":
                shapeTag = SVG_TAG_TRIANGLE;
                break;
            case "select-star":
                shapeTag = SVG_TAG_STAR;
                break;
        }

        // Add the color information to the tag
        colorTag = "style='fill:" + customColor + ";";
        shapeTag = shapeTag.replace("style='", colorTag);

        // Add the size and location information as well as the id information to the tag
        tempTag = "transform='matrix(" + shapeSize + " 0 0 " + shapeSize + " " + translateX + " " + translateY + ")' id='" + elemId + "'/>";
        shapeTag = shapeTag.replace(" />", tempTag);

        // Wrap the SVG string as an SVG object
        svgFragment = "<svg>" + shapeTag + "</svg>";

        // Add the SVG object to the temporary div element
        tempDiv.innerHTML = svgFragment;

        // Use Array.prototype.slice() method to copy items of tempDiv to the actual SVG board
        Array.prototype.slice.call(tempDiv.childNodes[0].childNodes).forEach(function(el) {
            document.querySelector("#" + boardId).appendChild(el);
            if (boardId === "svg-board") {
                el.addEventListener("touchmove", touchMoveHandler, false);
            }
        });

        // Update the index so that it can be used for the next shape to be added
        customIndex++;
    }

    /**
     * Handles events for touch start (Getting the first point)
     * @private
     * @param {Object} event - the object for click event
     */
    function touchStartHandler(event) {
        var touch = event.changedTouches[0];

        // Store information about the first touch point in the drawPath array
        drawPath[touch.identifier] = touch;
    }

    /**
     * Handles events for touch move (Moving shape around following the path of the touch points)
     * @private
     * @param {Object} event - the object for click event
     */
    function touchMoveHandler(event) {
        var i = 0,
            j = 0,
            x = 0,
            y = 0,
            touches = event.changedTouches,
            currentDrawPath = null,
            originalSize = 0,
            originalTransform = "",
            newTransform = "",
            transformArray = null;

        // While moving a shape, the color of the shape becomes translucent
        event.target.style.opacity = 0.5;

        // Move shape around following the path of the touch points
        for (i = 0; i < touches.length; i += 1) {
            currentDrawPath = drawPath[touches[i].identifier];

            if (currentDrawPath !== undefined) {
                // Get a string about the original transformation data of the shape object
                originalTransform = event.target.getAttribute("transform");

                // Split the string and insert each data into a temporary array so that each item can be accessed
                transformArray = originalTransform.split(" ");

                // Get size of the shape
                originalSize = Number(transformArray[3]);

                // Calculate x and y coordinates of the current touch point
                x = currentDrawPath.pageX - svgContainer.offsetLeft - (originalSize * 25);
                y = currentDrawPath.pageY - svgContainer.offsetTop - (originalSize * 25);

                // Insert the new coordinate data to the transformArray
                transformArray[4] = x;
                transformArray[5] = y + ")";

                // Create a string using the new data
                for (j = 0; j < transformArray.length; j++) {
                    newTransform += transformArray[j] + " ";
                }

                // Change the transformation data of the shape object with the new string
                event.target.setAttribute("transform", newTransform);

                drawPath[touches[i].identifier] = touches[i];
            }
        }

        // Indicate that a shape is being moved (not inserting a new shape)
        isMoving = 1;

        event.preventDefault();
    }

    /**
     * Handles events for touch end (Inserting shape to a new point)
     * @private
     * @param {Object} event - the object for click event
     */
    function touchEndHandler(event) {
        var i = 0,
            x = 0,
            y = 0,
            touch = event.changedTouches[0],
            originalSize = 0,
            originalTransform = "",
            newTransform = "",
            transformArray = null;

        // If not moving a shape, insert a new shape to the SVG board
        if (!isMoving) {
            // Get x and y coordinates of the final touch point
            x = touch.pageX - svgContainer.offsetLeft - (customSize * 25);
            y = touch.pageY - svgContainer.offsetTop - (customSize * 25);

            insertSvgShape("svg-board", "shape" + customIndex, customShape, customSize, x, y);

            delete drawPath[touch.identifier];
        }
        // If moving an existing shape, locate the shape to a new point at the end
        else {
            // Get a string about the original transformation data of the shape object
            originalTransform = event.target.getAttribute("transform");

            // Split the string and insert each data into a temporary array so that each item can be accessed
            transformArray = originalTransform.split(" ");

            // Get size of the shape
            originalSize = Number(transformArray[3]);

            // Calculate x and y coordinates of the final touch point
            x = touch.pageX - svgContainer.offsetLeft - (originalSize * 25);
            y = touch.pageY - svgContainer.offsetTop - (originalSize * 25);

            // At the end of moving a shape, restore the color opacity to the original state
            event.target.style.opacity = 1;

            // Insert the new coordinate data to the transformArray
            transformArray[4] = x;
            transformArray[5] = y + ")";

            // Create a string using the new data
            for (i = 0; i < transformArray.length; i++) {
                newTransform += transformArray[i] + " ";
            }

            // Change the transformation data of the shape object with the new string
            event.target.setAttribute("transform", newTransform);

            delete drawPath[touch.identifier];

            // Set the flag back to the initial state
            isMoving = 0;
        }
    }

    /**
     * Shows the pop-up page for shape selection
     * @private
     */
    function showShapePopup() {
        var i = 0;

        for (i = 0; i < shapeElements.length; i++) {
            shapeElements[i].style.opacity = 0.3;
        }

        // Show only the last selected shape in darker color
        document.querySelector("#" + customShape).style.opacity = 1.0;

        // Show the last selected size
        document.querySelector("#size-input").value = customSize;
        document.querySelector("#size-result").innerHTML = customSize;

        document.querySelector("#option-popup").style.visibility = "visible";

        // Change the flag
        isPopup = 1;
    }

    /**
     * Hides the pop-up page for shape selection
     * @private
     */
    function hideShapePopup() {
        document.querySelector("#option-popup").style.visibility = "hidden";

        // Set the flag back to the initial state
        isPopup = 0;
    }

    /**
     * Handles events for selecting (clicking) a shape in the pop-up page
     * @private
     * @param {Object} event - the object for click event
     */
    function selectShape(event) {
        var shapeClicked = event.target,
            i = 0;

        // Immediately apply the color change for shape selection
        for (i = 0; i < shapeElements.length; i++) {
            shapeElements[i].style.opacity = 0.3;
        }

        shapeClicked.style.opacity = 1.0;

        // Temporarily save the selection
        // In case the cancel button is clicked, the temporary selection can be disregarded
        customShapeTemp = shapeClicked.id;
    }

    /**
     * Applies change for shape selection
     * @private
     */
    function changeShape() {
        var shapeInput = document.querySelector("#shape-preview");

        customSize = customSizeTemp;
        customShape = customShapeTemp;

        // Remove the preview of the previous selection and insert the preview of the new selection
        shapeInput.removeChild(shapeInput.lastChild);
        insertSvgShape("shape-preview", "select-shape", customShape, 1, 0, 0);

        hideShapePopup();
    }

    /**
     * Applies change for color selection
     * @private
     */
    function changeColor(event) {
        var shapeInput = document.querySelector("#shape-preview");

        customColor = event.target.value;

        // Remove the preview of the previous selection and insert the preview of the new selection
        shapeInput.removeChild(shapeInput.lastChild);
        insertSvgShape("shape-preview", "select-shape", customShape, 1, 0, 0);
    }

    /**
     * Applies change for size selection
     * @private
     */
    function changeSize(event) {
        customSizeTemp = event.target.value;
        document.querySelector("#size-result").innerHTML = customSizeTemp;
    }

    /**
     * Clears the SVG board
     * @private
     */
    function clearSvgBoard() {
        // Remove all shape elements
        while (svgBoard.lastChild) {
            svgBoard.removeChild(svgBoard.lastChild);
        }
    }

    /**
     * Set default variables
     * @private
     */
    function setDefaultVariables() {
        // Set the default shape to be rectangle and the default size to be 1
        customShape = "select-rect";
        customSize = 1;
        customShapeTemp = customShape;
        customSizeTemp = customSize;

        // Insert the preview of the default shape selection
        insertSvgShape("shape-preview", "select-shape", customShape, 1, 0, 0);
    }

    /**
     * Set default event listeners
     * @private
     */
    function setDefaultEvents() {
        var i = 0;

        // Add event listeners forSVG board
        svgBoard.addEventListener("touchstart", touchStartHandler, false);
        svgBoard.addEventListener("touchend", touchEndHandler, false);

        // Add event listeners forPop-up page
        document.querySelector("#shape-input").addEventListener("click", showShapePopup, false);
        document.querySelector("#shape-cancel-btn").addEventListener("click", hideShapePopup, false);
        document.querySelector("#shape-set-btn").addEventListener("click", changeShape, false);
        document.querySelector("#color-input").addEventListener("change", changeColor, false);
        document.querySelector("#size-input").addEventListener("change", changeSize, false);
        document.querySelector("#clear-input").addEventListener("click", clearSvgBoard, false);

        for (i = 0; i < shapeElements.length; i++) {
            shapeElements[i].addEventListener("click", selectShape);
        }

        // Add event listener for tizenhwkey
        window.addEventListener("tizenhwkey", function onTizenHwKey(event) {
            if (event.keyName === "back") {
                if (isPopup === 1) {
                    hideShapePopup();
                } else {
                    try {
                        tizen.application.getCurrentApplication().exit();
                    } catch (err) {
                        console.error("Error: ", err);
                    }
                }
            }
        });

    }

    /**
     * Initiates the application
     * @private
     */
    function init() {
        setDefaultVariables();
        setDefaultEvents();
    }

    window.onload = init;
}());